#import "ZegoZpnsPlugin.h"
#import <ZPNs/ZPNs.h>
#import "NSMutableDictionary+safeInvoke.h"
#import "NSDictionary+safeInvoke.h"
@interface ZegoZpnsPlugin ()<ZPNsNotificationCenterDelegate>

@property (nonatomic, strong) FlutterEventChannel *eventChannel;

@property (nonatomic, strong) FlutterEventSink events;

@property bool debug;

@end

@implementation ZegoZpnsPlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
  FlutterMethodChannel* channel = [FlutterMethodChannel
      methodChannelWithName:@"zego_zpns"
            binaryMessenger:[registrar messenger]];
  ZegoZpnsPlugin* instance = [[ZegoZpnsPlugin alloc] init];
  [registrar addApplicationDelegate:instance];
  [registrar addMethodCallDelegate:instance channel:channel];
  [[ZPNs shared] setZPNsNotificationCenterDelegate:(id)instance];
  FlutterEventChannel *eventChannel = [FlutterEventChannel eventChannelWithName:@"zpns_event_handler" binaryMessenger:[registrar messenger]];
  [eventChannel setStreamHandler:(id)instance];
  instance.eventChannel = eventChannel;
}

- (FlutterError* _Nullable)onListenWithArguments:(id _Nullable)arguments
                                       eventSink:(FlutterEventSink)events {
    self.events = events;
    
    return nil;
}

- (FlutterError* _Nullable)onCancelWithArguments:(id _Nullable)arguments {
    self.events = nil;
    return nil;
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
  if ([@"getPlatformVersion" isEqualToString:call.method]) {
    result([@"iOS " stringByAppendingString:[[UIDevice currentDevice] systemVersion]]);
  }
  else if ([@"getVersion" isEqualToString:call.method]){
      
      result([ZPNs getVersion]);
  }
  else if ([@"applyNotificationPermission" isEqualToString:call.method]){
      
      if (@available(iOS 10.0, *)) {
          UNUserNotificationCenter *center = [UNUserNotificationCenter currentNotificationCenter];
          center.delegate = (id)[ZPNs shared];
          [center requestAuthorizationWithOptions:(UNAuthorizationOptionAlert | UNAuthorizationOptionBadge | UNAuthorizationOptionSound) completionHandler:^(BOOL granted, NSError * _Nullable error) {
              if (granted) {
                  //点击允许
                  [center getNotificationSettingsWithCompletionHandler:^(UNNotificationSettings * _Nonnull settings) {
                     
                  }];

              } else {
                  
              }
          }];
      }
      else if ([[UIDevice currentDevice].systemVersion floatValue] >8.0){
          //iOS8 - iOS10
          [[UIApplication sharedApplication] registerUserNotificationSettings:[UIUserNotificationSettings settingsForTypes:UIUserNotificationTypeAlert | UIUserNotificationTypeSound | UIUserNotificationTypeBadge categories:nil]];

      }
      else {
          // Fallback on earlier versions

      }
      result(nil);
      
  }
  else if ([@"enableDebug" isEqualToString:call.method]){
      self.debug = [[call.arguments objectForKey:@"debug"] boolValue];
  }
  else if ([@"getPushConfig" isEqualToString:call.method]){
      
  }
  else if ([@"setPushConfig" isEqualToString:call.method]){

  }
  else if ([@"registerPush" isEqualToString:call.method]){
      [[ZPNs shared] registerAPNs];
      result(nil);
  }
  else if([@"addLocalNotification" isEqualToString:call.method]){
      UNUserNotificationCenter *center = [UNUserNotificationCenter currentNotificationCenter];
      UNMutableNotificationContent *content = [[UNMutableNotificationContent alloc] init];
      NSDictionary *localMessageDic = [call.arguments safeObjectForKey:@"message"];
      content.title = [localMessageDic safeObjectForKey:@"title"];
      content.body = [localMessageDic safeObjectForKey:@"content"];
      NSMutableDictionary *userInfo = [[NSMutableDictionary alloc] init];
      [userInfo safeSetObject:[localMessageDic safeObjectForKey:@"payload"] forKey:@"payload"];
      content.userInfo = userInfo;
      
      NSString *sound = [localMessageDic safeObjectForKey:@"iOSSound"];
      if(sound != nil && ![sound isEqual:@""]){
          content.sound = [UNNotificationSound soundNamed:sound];
      }
      #if __IPHONE_OS_VERSION_MAX_ALLOWED >= 150000
          if (@available(iOS 15.0, *)) {
              content.interruptionLevel = UNNotificationInterruptionLevelActive;
          }
      #endif
      UNNotificationRequest *request = [UNNotificationRequest requestWithIdentifier:@"" content:content trigger:nil];
      [center addNotificationRequest:request withCompletionHandler:^(NSError * _Nullable error) {
          
      }];
      result(nil);
  }
  else if ([@"createNotificationChannel" isEqualToString:call.method]){
      result(nil);
  }
  else {
    result(FlutterMethodNotImplemented);
  }
}

#pragma mark - ZPNsNotificationCenterDelegate

- (void)onRegistered:(NSString *)Pushid{
    if(_events == nil){
        return;
    }
    NSDictionary *resultDic = @{@"method":@"onRegistered",@"message":@{@"pushID":Pushid,@"errorCode":[NSNumber numberWithInt:0],@"pushSourceType":[NSNumber numberWithInt:10],@"msg":@"",@"commandResult":@""}};
    _events(resultDic);
}



- (void)ZPNsNotificationCenter:(UNUserNotificationCenter *)center
       willPresentNotification:(UNNotification *)notification
                      userInfo:(NSDictionary *)userInfo
         withCompletionHandler:(void (^)(UNNotificationPresentationOptions))completionHandler
API_AVAILABLE(macos(10.14), ios(10.0)){
    if(_events == nil){
        return;
    }
    NSMutableDictionary *messageMap = [[NSMutableDictionary alloc] init];
    [messageMap setObject:notification.request.content.title forKey:@"title"];
    [messageMap setObject:notification.request.content.body forKey:@"content"];
    [messageMap setObject:userInfo forKey:@"extras"];
    [messageMap setObject:[NSNumber numberWithInt:10] forKey:@"pushSourceType"];
    
    NSDictionary *resultDic = @{@"method":@"onNotificationArrived",@"message":messageMap};
    _events(resultDic);
}

- (void)ZPNsNotificationCenter:(UNUserNotificationCenter *)center
    didReceiveNotificationResponse:(UNNotificationResponse *)response
                          userInfo:(NSDictionary *)userInfo
         withCompletionHandler:(void (^)(void))completionHandler API_AVAILABLE(macos(10.14), ios(10.0)){
    if(_events == nil){
        return;
    }
    NSMutableDictionary *messageMap = [[NSMutableDictionary alloc] init];
    
    [messageMap setObject:response.notification.request.content.title forKey:@"title"];
    [messageMap setObject:response.notification.request.content.body forKey:@"content"];
    [messageMap setObject:userInfo forKey:@"extras"];
    [messageMap setObject:[NSNumber numberWithInt:10] forKey:@"pushSourceType"];
    
    NSDictionary *resultDic = @{@"method":@"onNotificationClicked",@"message":messageMap};
    _events(resultDic);
}
#pragma mark - AppDelegate

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
    
    return YES;
}

- (void)application:(UIApplication *)application didRegisterForRemoteNotificationsWithDeviceToken:(NSData *)deviceToken {
    [[ZPNs shared] setDeviceToken:deviceToken isProduct:!self.debug];
}

- (void)application:(UIApplication*)application
didFailToRegisterForRemoteNotificationsWithError:(NSError*)error{
    
    NSDictionary *resultDic = @{@"method":@"onRegistered",@"message":@{@"pushID":@"",@"errorCode":[NSNumber numberWithInteger:error.code],@"pushSourceType":[NSNumber numberWithInt:10],@"msg":error.localizedDescription?error.localizedDescription:@"",@"commandResult":@""}};
    _events(resultDic);
}



@end
