import 'package:flutter/services.dart';
import 'package:zego_zpns/src/internal/zpns_converter.dart';
import 'package:zego_zpns/src/internal/zpns_event_handler_impl.dart';
import 'package:zego_zpns/zego_zpns.dart';
import 'dart:io';

class ZPNsEngine implements ZPNs {
  static const MethodChannel _channel = MethodChannel('zego_zpns');

  ZPNsEngine() {
    ZPNsEventHandlerImpl.registerEventHandler();
  }

  @override
  Future<void> registerPush({ZPNsIOSEnvironment? iOSEnvironment}) async {
    if (Platform.isIOS || Platform.isMacOS) {
      if (iOSEnvironment == null) {
        throw PlatformException(
            code: "1",
            message:
                "ZPNsIOSEnvironment can not be Unkown, You must pass in ZPNsIOSEnvironment when invoke registerPush method on iOS platform");
      }
      switch (iOSEnvironment) {
        case ZPNsIOSEnvironment.Development:
          _channel.invokeMethod('enableDebug', {'debug': true});
          break;
        case ZPNsIOSEnvironment.Production:
          _channel.invokeMethod('enableDebug', {'debug': false});
          break;
      }
    }
    return await _channel.invokeMethod('registerPush');
  }

  // @override
  // Future<ZPNsConfig> getPushConfig() async {
  //   Map resultMap = await _channel.invokeMethod('getPushConfig');
  //   return ZPNsConverter.cnvZPNsConfigMapToObject(resultMap['config']);
  // }
  @override
  Future<void> applyNotificationPermission() async {
    return await _channel.invokeMethod('applyNotificationPermission');
  }

  @override
  Future<void> addLocalNotification(ZPNsLocalMessage message) async {
    return await _channel.invokeMethod('addLocalNotification',
        {'message': ZPNsConverter.cnvZPNsLocalMessageObjectToMap(message)});
  }

  @override
  Future<void> createNotificationChannel(
      ZPNsNotificationChannel channel) async {
    return await _channel.invokeMethod('createNotificationChannel',
        {'channel': ZPNsConverter.cnvZPNsNotificationChannelToMap(channel)});
  }
}
